using System;
using System.Collections.Generic;
using System.Linq;
using MacGruber;
using SimpleJSON;
using UnityEngine;

namespace CheesyFX
{
    public class TouchMe : MVRScript
    {
        public static TouchMe singleton;
        public static string packageUid;
        private UnityEventsListener uiListener;

        // private SlapHandler slapHandler;
        

        private List<object> UIElements = new List<object>();
        private UIDynamicTabBar tabbar;
        
        public static List<Atom> excludedAtoms = new List<Atom>();
        private static JSONStorableString excludedInfo = new JSONStorableString("excludeInfo", "<b>Excluded Atoms</b>\n");

        private static JSONStorableStringChooser excludedChooser =
            new JSONStorableStringChooser("atomChooser", null, "", "... this Atom");

        public override void Init()
        {
            singleton = this;
            packageUid = Utils.GetPackagePath(this);
            UIManager.SetScript(this, CreateUIElement, leftUIElements, rightUIElements);
            Utils.OnInitUI(CreateUIElement);

            SlapHandler.Init();
            NippleManager.Init();
            
            excludedChooser.choices = SuperController.singleton.GetAtomUIDs();
            SuperController.singleton.onAtomAddedHandlers += AddExclusionChoice;
            SuperController.singleton.onAtomUIDsChangedHandlers += SyncAtomNames;
            
            RegisterString(excludedInfo);
            excludedInfo.isRestorable = false;
            RegisterStringChooser(excludedChooser);
            
            ExcludeDefaults();
            CreateUI();
        }
        
        public override void InitUI()
        {
            base.InitUI();
            if(UITransform == null || uiListener != null) return;
            uiListener = UITransform.gameObject.AddComponent<UnityEventsListener>();
            uiListener.onEnabled.AddListener(() => UIManager.SetScript(this, CreateUIElement, leftUIElements, rightUIElements));
        }

        public static void OnEnter(TouchZone touchZone, Collision collision)
        {
            ContactPoint contactPoint = collision.contacts[0];
            Vector3 velocity = collision.relativeVelocity;
            float intensity = Math.Abs(Vector3.Dot(velocity, contactPoint.normal));
            if (intensity >= SlapHandler.sexSlapThreshold) SlapHandler.Slap(touchZone, collision, intensity, contactPoint);
            if (touchZone.name == "lNipple" || touchZone.name == "lAreola")
            {
                // intensity.Print();
                NippleManager.Stimulate(.1f*intensity, 0);
            }
            else if (touchZone.name == "rNipple" || touchZone.name == "rAreola")
            {
                NippleManager.Stimulate(.025f, 1);
            }
        }

        public static void OnStay(TouchZone touchZone)
        {
            if (touchZone.name == "lNipple" || touchZone.name == "lAreola")
            {
                NippleManager.Stimulate(.05f*Time.fixedDeltaTime, 0);
                // touchZone.name.Print();
            }
            else if(touchZone.name == "rNipple" || touchZone.name == "rAreola") NippleManager.Stimulate(Time.fixedDeltaTime, 1);
        }

        private void OnDestroy()
        {
            foreach(TouchZone touchZone in BodyRegionMapping.touchZones.Values.ToList()){
                touchZone.Destroy();
            }
            AudioImporter.UnloadBundles();
            SlapHandler.Destroy();
            SuperController.singleton.onAtomAddedHandlers -= AddExclusionChoice;
            SuperController.singleton.onAtomUIDsChangedHandlers -= SyncAtomNames;
        }

        public override void LateRestoreFromJSON(
            JSONClass jc,
            bool restorePhysical,
            bool restoreAppearance,
            bool setMissingToDefault
        )
        {
            if (jc.HasKey("excludeInfo"))
            {
                ResetExcluded();
                var uidsToIgnore = ((string)jc["excludeInfo"]).Split(new [] {"\n"}, StringSplitOptions.None).Skip(1);
                foreach (var uid in uidsToIgnore)
                {
                    Exclude(uid, true);
                }
            }
        }
        
        private void CreateUI()
        {
            // UIManager.SetScript(this, CreateUIElement, leftUIElements, rightUIElements);
            UIManager.RemoveUIElements(UIElements);
            tabbar = UIManager.CreateTabBar(new [] {"General", "Slaps", "Nipples", "TouchTriggers", "SlapTriggers"}, SelectTab, script:this);
            tabbar.SelectTab(lastTabId);
        }

        private int lastTabId;
        public void ClearUI()
        {
            UIManager.RemoveUIElements(leftUIElements.Select(x => (object) x.GetComponent<UIDynamic>()).ToList());
            UIManager.RemoveUIElements(rightUIElements.Select(x => (object) x.GetComponent<UIDynamic>()).ToList());
        }

        private static List<object> CreateGeneralUI()
        {
            List<object> UIElements = new List<object>();
            var button = TouchMe.singleton.CreateButton("Disable Touches with...");
            button.button.onClick.AddListener(() => Exclude(excludedChooser.val, true));
            UIElements.Add(button);
	        
            button = TouchMe.singleton.CreateButton("Enable Touches with...");
            button.button.onClick.AddListener(() => Exclude(excludedChooser.val, false));
            UIElements.Add(button);
	        
            excludedChooser.CreateUI(UIElements:UIElements, rightSide: true, chooserType:2);
            button = TouchMe.singleton.CreateButton("Reset");
            button.button.onClick.AddListener(ResetExcluded);
            UIElements.Add(button);
	        
            var textfield = TouchMe.singleton.CreateTextField(excludedInfo, true);
            textfield.height = 400f;
            UIElements.Add(textfield);
            return UIElements;
        }

        private void SelectTab(int id)
        {
            lastTabId = id;
            UIManager.RemoveUIElements(UIElements);
            if(id == 0) UIElements = CreateGeneralUI();
            else if(id == 1) UIElements = SlapHandler.CreateUI();
            else if (id == 2) UIElements = NippleManager.CreateUI();
            else if (id == 3) UIElements = TouchTriggers.CreateUI();
            else UIElements = SlapTriggers.CreateUI();
        }
        
        public static void Exclude(string uid, bool val)
        {
            Atom atom = SuperController.singleton.GetAtomByUid(uid);
            if (atom == null) return;
            if (val)
            {
                if(!excludedAtoms.Contains(atom))
                {
                    excludedAtoms.Add(atom);
                    excludedInfo.val += atom.uid + "\n";
                }
            }
            else if(excludedAtoms.Remove(atom))
            { 
                excludedInfo.val = "<b>Excluded Atoms</b>\n"+string.Join("\n", excludedAtoms.Select(x => x.uid).ToArray());
            }
        }
        
        private static void AddExclusionChoice(Atom atom)
        {
            if (excludedChooser.choices.Contains(atom.uid)) return;
            var choices = excludedChooser.choices;
            choices.Insert(0, atom.uid);
            excludedChooser.choices = new List<string>(choices);
        }
        
        private static void ExcludeDefaults()
        {
            string[] typesToExclude = {"Panel", "Slate", "Wall", "Glass", "CustomUnityAsset", "Apt", "Cyberpunk", "Deco"};
            foreach (var atom in SuperController.singleton.GetAtoms())
            {
                if(typesToExclude.Any(x => atom.type.Contains(x))) Exclude(atom.uid, true);
            }
        }
        
        private static void SyncAtomNames(List<string> uids)
        {
            excludedChooser.choices = new List<string>(uids);
            var ignoredUids = excludedAtoms.Select(x => x.uid).ToArray();
            excludedInfo.val = "<b>Excluded Atoms</b>\n"+string.Join("\n", ignoredUids);
        }
        
        public static void ResetExcluded()
        {
            excludedAtoms.Clear();
            excludedInfo.val = "<b>Excluded Atoms</b>\n";
        }
    }
}